=begin

= apache/asp-run.rb

Copyright (C) 2003 Gregoire Lejeune <gregoire.lejeune@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

== Overview

Apache::AspRun handles eRuby files by Asp.

== Example of .htaccess

AuthName "Ruby/ASP"

RubySetEnv Global /var/www/tests/ruby-asp/global-asa

RubyRequire apache/aspruby-run
<Files ~ (\.asp)>
  SetHandler ruby-object
  RubyHandler Apache::AspRun.instance
</Files>

=end

require "singleton"
require "tempfile"
require "asp/common"
require "asp/compile"
require "asp/globalasa"
require "asp/response"
require "asp/request"
require "asp/session"
require "asp/application"
require "asp/server"

module Apache
  class AspRun
    include Singleton

    def handler(r)
      if r.method_number == M_OPTIONS
        r.allowed |= (1 << M_GET)
        r.allowed |= (1 << M_POST)
        return DECLINED
      end
      if r.finfo.mode == 0
        return NOT_FOUND
      end

      s = r.server( )
#      c = r.connection( )
#      t = r.headers_in( )
      
      r.setup_cgi_env
      
      btime = Time::now
      
	  ## Add global-asa path to library search path
	  #$:.unshift( File.join( ENV["Global"] ) )

	  ## Load global.asa
      oGAsa = Asp::GlobalASA::new( ENV["Global"], r ) 
      if oGAsa.getAsaCode == false 
        print __FILE__," L",__LINE__," :: Call ASP.Error\n";
        return OK
      end

      $Request = Asp::Request::new( r )
      $Response = Asp::Response::new( r )
      $Session = Asp::Session::new( r )
      $Server = Asp::Server::new( r )

      filename = r.filename.dup
      filename.untaint
      
	  run_state = 0
      code = compile(filename)
	  begin
	    run_state = 1
        prerun(r)
	    run_state = 2
        run(code, filename)
	    run_state = 3
        postrun(r, btime)
	  rescue NameError, SyntaxError, ZeroDivisionError, RuntimeError, ArgumentError => e
	    debug(e, r, code, filename, run_state)
	  end

      return OK
    end

    private

    def initialize
      @compiler = AspCompiler.new
      @compiler.put_cmd = 'print'
    end

    def compile(filename)
      open(filename) do |f|
        return @compiler.compile(f.read)
      end
    end

    def prerun(r)
      if self.private_methods.member?( "script_OnStart" ) == true
        script_OnStart
      end
      Apache.chdir_file(r.filename)
    end

    def run(code, filename)
      tmpdir = (ENV['TMPDIR'] || ENV['TMP'] || ENV['TEMP'] || '/tmp').untaint
      file = Tempfile.new(File.basename(filename) + ".", tmpdir)
      begin
        file.print(code)
        file.close
        load(file.path, true)
      ensure
        file.close(true)
      end
    end

    def postrun(r, btime)
      if self.private_methods.member?( "script_OnEnd" ) == true
        script_OnEnd

        etime = Time::now
        ctime = etime - btime
        print "<!-- Ruby/ASP v",Asp::Common::version," served page in ",ctime," seconds -->"
      end

      r.content_type = format( "%s; charset=%s", $Response.contentType, $Response.charset )
      $Response.getHeader.each_pair{ |key,value|
        r.headers_out[key] = value
      }
      $Response.addCookiesHeader.each { |cookie|
        r.headers_out.add( "Set-Cookie", cookie )
      }
      sessionCookie = $Session.addSessionCookieHeader
      if sessionCookie.nil? == false
        r.headers_out.add( "Set-Cookie", sessionCookie )
      end
      r.send_http_header
    end

	def debug(e, r, code, filename, run_state)
	  print "<B>Ruby/Asp v",Asp::Common::version,"</B><BR>\n"
	  print "ERROR State #",run_state,":<BR>\n"
	  print "<B>",r.escape_html(e.message),"</B><BR>\n"
	  print "File ",filename,"<BR>\n"
	  print "<HR><TABLE STYLE='spacing: 0px; padding: 8px'>\n"
	  i = 0
	  r.escape_html(code).each_line { |l|
	    i += 1
	    printf( "<TR><TD STYLE='border-right: 1px solid #000000'><b>%04d</b>&nbsp;</TD><TD>&nbsp;%s</TD></TR>\n", i, l.chomp )
	  }
	  print "</TABLE>\n"
	end

  end
end
