# Copyright (C) 2003 Gregoire Lejeune <gregoire.lejeune@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

module Asp
  class Session
    require 'asp/common'
	require 'gdbm'
    require 'ftools'
    
    public

	def [](key)
      if @sessionID.nil? == false
	    return @sessionDB[key]
      end

      return nil
	end
	
	def []=(key, val)
      if @sessionID.nil? == true
        @sessionID = newSessionID
        openSessionDB
      end

      @sessionDB[key] = String( val )
	end

    def timeout( min )
      @bSendCookie = true
      @timeout = Time::now+(60*min)
    end

	def sessionID
	  return @sessionID
	end

	def abandon
      if @sessionID.nil? == false
        @sessionID = nil
        @sessionDB.clear
        @sessionDB.close
        @sessionDB = nil
        File::rm_f( @sessionDBFile )
		@bSendCookie = true
		@timeout = Time::now
      end
	end
    
    ## -- Reserved for Ruby/ASP internal use --
    def addSessionCookieHeader
	  if @sessionDB.nil? == false
	    @sessionDB.close
	  end
	  
      if @sessionID.nil? == false and @bSendCookie == true
        expires = "; expires="+String( @timeout )
        cookie = Asp::Common::sessionID+"="+@sessionID+expires+"; path=/"
        return cookie
      end

      return nil
    end

    private 
	
    def initialize( r, id=nil )
      @oApacheRequest = r
      @sessionID = nil
      @sessionDBFile = nil
      @sessionDB = nil
      @timeout = Time::now+(60*5)
      @bSendCookie = true

      if id.nil? == false
        @sessionID = id
      else
        getSessionIDFromCookie
      end

      openSessionDB
    end

    def openSessionDB
      if @sessionDB.nil? == true and @sessionID.nil? == false
        @sessionDBFile = "/tmp/"+@sessionID+".00"
		secure = @sessionDBFile.taint
		$SAFE=1
		secure.untaint
        @sessionDB = GDBM::open( @sessionDBFile ) 
      end
    end

    def getSessionIDFromCookie
      cookiesStr = @oApacheRequest.headers_in['Cookie']
     
      if cookiesStr.nil? == false
        parts = cookiesStr.split( /;\s*/ )
        parts.each { |v|
          name, value = v.split( /\=/, 2 )
          name = Asp::Common::url_decode( name )
          next if name != Asp::Common::sessionID
          @sessionID = value
          @bSendCookie = false
          break
        }
      end
    end

    def newSessionID
      require 'digest/md5'
      @bSendCookie = true

      md5 = Digest::MD5::new
      md5.update(String(Time::now))
      md5.update(String(rand(0)))
      md5.update(String($$))
      md5.update('foobar')
      md5.hexdigest[0,16]
    end

  end
end
